# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: example.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\rexample.proto\x12\x13grpc_wrappers.tests\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/wrappers.proto"\xbe\x05\n\x0e\x45xampleMessage\x12\x12\n\nbool_value\x18\x01 \x01(\x08\x12\x11\n\tint_value\x18\x02 \x01(\x03\x12\x14\n\x0cstring_value\x18\x03 \x01(\t\x12\x34\n\nenum_value\x18\x04 \x01(\x0e\x32 .grpc_wrappers.tests.ExampleEnum\x12\x37\n\nsubmessage\x18\x05 \x01(\x0b\x32#.grpc_wrappers.tests.ExampleMessage\x12#\n\x17\x64\x65precated_string_value\x18\x06 \x01(\tB\x02\x18\x01\x12\x12\n\nint_values\x18\x07 \x03(\x03\x12\x35\n\x0b\x65num_values\x18\x08 \x03(\x0e\x32 .grpc_wrappers.tests.ExampleEnum\x12\x38\n\x0bsubmessages\x18\t \x03(\x0b\x32#.grpc_wrappers.tests.ExampleMessage\x12L\n\x11nested_submessage\x18\n \x01(\x0b\x32\x31.grpc_wrappers.tests.ExampleMessage.NestedMessage\x12\x36\n\x12wrapped_bool_value\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x38\n\x13wrapped_float_value\x18\x0c \x01(\x0b\x32\x1b.google.protobuf.FloatValue\x12:\n\x14wrapped_string_value\x18\r \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x35\n\x11wrapped_timestamp\x18\x0e \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a#\n\rNestedMessage\x12\x12\n\nbool_value\x18\x01 \x01(\x08*0\n\x0b\x45xampleEnum\x12\x05\n\x01\x41\x10\x00\x12\x05\n\x01\x42\x10\x01\x12\x05\n\x01\x43\x10\x02\x12\x05\n\x01\x44\x10\x03\x12\x05\n\x01\x45\x10\x04\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "example_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _EXAMPLEMESSAGE.fields_by_name["deprecated_string_value"]._options = None
    _EXAMPLEMESSAGE.fields_by_name[
        "deprecated_string_value"
    ]._serialized_options = b"\030\001"
    _globals["_EXAMPLEENUM"]._serialized_start = 808
    _globals["_EXAMPLEENUM"]._serialized_end = 856
    _globals["_EXAMPLEMESSAGE"]._serialized_start = 104
    _globals["_EXAMPLEMESSAGE"]._serialized_end = 806
    _globals["_EXAMPLEMESSAGE_NESTEDMESSAGE"]._serialized_start = 771
    _globals["_EXAMPLEMESSAGE_NESTEDMESSAGE"]._serialized_end = 806
# @@protoc_insertion_point(module_scope)
